import { fetch } from '@whatwg-node/fetch';

function createGenericSDK({ endpoint, fetchFn = fetch } = {}) {
    return new Proxy({}, {
        get(_target, path) {
            return new Proxy({}, {
                get(_target, method) {
                    return function (requestParams) {
                        var _a, _b;
                        const url = new URL(path, endpoint);
                        for (const pathParamKey in requestParams.params || {}) {
                            const value = (_a = requestParams.params) === null || _a === void 0 ? void 0 : _a[pathParamKey];
                            if (value) {
                                url.pathname = url.pathname.replace(`{${pathParamKey}}`, value);
                            }
                        }
                        for (const queryParamKey in requestParams.query || {}) {
                            const value = (_b = requestParams.query) === null || _b === void 0 ? void 0 : _b[queryParamKey];
                            if (value) {
                                if (Array.isArray(value)) {
                                    value.forEach(v => url.searchParams.append(queryParamKey, v));
                                }
                                else {
                                    url.searchParams.append(queryParamKey, value);
                                }
                            }
                        }
                        const requestInit = {
                            method,
                            headers: requestParams.headers || {},
                        };
                        if (requestParams.json) {
                            requestInit.body = JSON.stringify(requestParams.json);
                            requestInit.headers['Content-Type'] = 'application/json';
                        }
                        return fetchFn(url, requestInit);
                    };
                },
            });
        },
    });
}

export { createGenericSDK };
